/*
 *      Program do przegladania logow z ekg.
 *      Linia komend.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CMDLINE_HH__
#define __CMDLINE_HH__

extern "C" {
	#include <ncurses.h>
}

#include <string>
#include <map>
#include <list>
#include <vector>
#include <cctype>


/// Para klawiszy; 0 oznacza dowolny
struct key_pair
{
	int k1, k2;
	key_pair(int a, int b) : k1(a),k2(b) {}

	/// Jesli ktorys z klawiszy jest == 0, to porownanie jest zbedne
	bool operator == (const key_pair & kp) const
	{
		return  ((k1==0 || kp.k1==0) ? true : (k1==kp.k1)) &&
			((k2==0 || kp.k2==0) ? true : (k2==kp.k2));
	}

	/// Zeby mozna bylo uzywac jako klucza w mapie
	bool operator < (const key_pair & kp) const
	{
		// jesli pierwszy znak jest dowolny, to porownujemy _tylko_ drugie
		if(!k1 || !kp.k1)
			return k2<kp.k2;

		// w przeciwnym wypadku porownujemy i pierwsze i drugie
		return (k2==kp.k2) ? (k1<kp.k1) : (k2<kp.k2);
	}

	/// Powoduje przesuniecie k2 na k1 i wstawienie x na k2
	void add(int x)
	{
		k1 = k2;
		k2 = x;
	}
};



/**
 * @brief Klasa linii komend.
 *
 * Klasa linii komend z interpreterem klawiatury, wyswietlaniem na ekran
 * wiersza polecen. Niektore klawisze, zadane jako specjalne, nie sa
 * interpretowane jako czesc linii polecen, ale przekazywane na zewnatrz
 * jako zdarzenie.
 *
 * Gdy linia komend jest aktywna (set_active(true)), to klawisze KEY_UP,
 * KEY_DOWN, KEY_LEFT, KEY_RIGHT, TAB, ENTER, HOME, DEL, END, BACKSPACE,
 * PGUP i PGDOWN nie sa przekazywane jako komenda (get_cmd), ale sa
 * interpretowane jako sterowanie linia komend.
 */
class cmdline_t {
public:
	enum type_t { PARSE_ERROR, WRONG_END };

	// wyjatki
	struct cmdline_error_t {
		cmdline_error_t(type_t atype, char ach):type(atype),ch(ach) {}

		type_t type;
		char ch;
	};


	/// Konstruktor: przyjmuje szerokosc oraz okno ncurses.
	cmdline_t(unsigned int width, WINDOW * win=0);

	/// Zwraca ciag z trescia komendy lub inf. o klawiszu specjalnym.
	const std::string get_cmd();
	/// Ustawia aktywnosc: ma to wplyw na przechwytywanie strzalek.
	void set_active(bool a);
	/// Wypisuje sie na ekranie
	void print();
	/// dodaje klawisz do specjalnych
	void register_key(int key, const std::string & descr);
	/// dodaje sekwencje do specjalnych
	void register_sequence(int key1, int key2, const std::string & descr);
	/// podaje argumenty dla ostatniej komendy
	const std::vector<std::string> & argv();
	/// zwraca true, jesli kursor stoi na koncu linii
	bool is_good_to_complete();
	/// zwraca ostatni argument z linii polecen
	std::string get_last();
	/// zwraca tresc komendy
	std::string get_first();
	/// sprawdza, na ile czesci mozna sparsowac linie; jesli na jedna zwaraca true
	bool one_piece();
	/// dodaje na koncu linii string
	void add_completition(const std::string & str) { _line += str+" "; _line_pos = _line.size(); }
	/// zwraca dwa ostatnio nacisniete klawisze
	const key_pair & get_keys() { return _kp; }

private:

	/// parsuje podany string, komende wstawia do _command, argumenty do _argv
	void parse(const std::string & str);
	/// pobiera ze stringa jedna czesc pomiedzy bialymi znakami; zaczyna od litery i
	std::string parse_get_piece(const std::string & str, unsigned int & i);

	/// Okno ncurses dla linii polecen
	WINDOW * _win;
	/// Szerokosc okna
	unsigned int _width;
	/// Tresc linii polecen.
	std::string _line;
	/// Gdzie jest kursor.
	unsigned int _line_pos;
	/// klawisze specjalne: kod + id
	std::map<key_pair, std::string> _spc_keys;
	/// Czy linia komend jest wybrana jako aktywna
	bool _active;
	/// maksymalna dlugosc historii
	unsigned int _history_size;
	/// czy weszlismy do historii
	bool _history_used;
	/// iterator na aktualna linie
	std::list<std::string>::iterator _history_iter;
	/// historia polecen
	std::list<std::string> _history;
	/// komenda - po sparsowaniu
	std::string _command;
	/// wektor argumentow dla komend
	std::vector<std::string> _argv;
	/// informacja o ostatnio odebranych klawiszach
	key_pair _kp;
};

#endif /* __CMDLINE_HH__ */
